"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkEventsContinuity = exports.emptyLoadEventsResult = exports.threadCounterToBuffer = exports.threadCounterHexStringToBuffer = exports.hexStringToThreadCounter = exports.threadCounterToHexString = exports.cursorToThreadArray = exports.threadArrayToCursor = exports.initThreadArray = void 0;

var _assert = _interopRequireDefault(require("assert"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const checkThreadArrayLength = threadArray => {
  _assert.default.strictEqual(threadArray.length, _constants.THREAD_COUNT, 'Cursor must be represented by array of 256 numbers');
};

const checkThreadCounterHexString = threadCounter => {
  _assert.default.strictEqual(threadCounter.length, _constants.THREAD_COUNTER_BYTE_LENGTH * 2, 'Wrong length of threadCounter in hex form');
};

const initThreadArray = () => {
  const threadCounters = new Array(_constants.THREAD_COUNT);
  threadCounters.fill(0);
  return threadCounters;
};

exports.initThreadArray = initThreadArray;

const threadArrayToCursor = threadArray => {
  checkThreadArrayLength(threadArray);
  const cursorBuffer = Buffer.alloc(_constants.CURSOR_BUFFER_SIZE);

  for (let i = 0; i < threadArray.length; ++i) {
    const threadCounter = threadArray[i];
    const threadCounterBuffer = threadCounterToBuffer(threadCounter);
    threadCounterBuffer.copy(cursorBuffer, i * _constants.THREAD_COUNTER_BYTE_LENGTH);
  }

  return cursorBuffer.toString('base64');
};

exports.threadArrayToCursor = threadArrayToCursor;

const cursorToThreadArray = cursor => {
  if (cursor == null) return initThreadArray();
  const cursorBuffer = Buffer.from(cursor, 'base64');

  _assert.default.strictEqual(cursorBuffer.length, _constants.CURSOR_BUFFER_SIZE, 'Wrong size of cursor buffer');

  const threadCounters = new Array(_constants.THREAD_COUNT);

  for (let i = 0; i < cursorBuffer.length / _constants.THREAD_COUNTER_BYTE_LENGTH; i++) {
    const hexString = cursorBuffer.slice(i * _constants.THREAD_COUNTER_BYTE_LENGTH, (i + 1) * _constants.THREAD_COUNTER_BYTE_LENGTH).toString('hex');
    checkThreadCounterHexString(hexString);
    threadCounters[i] = hexStringToThreadCounter(hexString);
  }

  return threadCounters;
};

exports.cursorToThreadArray = cursorToThreadArray;

const threadCounterToHexString = threadCounter => threadCounter.toString(16).padStart(_constants.THREAD_COUNTER_BYTE_LENGTH * 2, '0');

exports.threadCounterToHexString = threadCounterToHexString;

const hexStringToThreadCounter = threadCounter => {
  checkThreadCounterHexString(threadCounter);
  const num = parseInt(threadCounter, 16);
  (0, _assert.default)(!Number.isNaN(num));
  return num;
};

exports.hexStringToThreadCounter = hexStringToThreadCounter;

const threadCounterHexStringToBuffer = threadCounter => {
  checkThreadCounterHexString(threadCounter);
  const buffer = Buffer.alloc(_constants.THREAD_COUNTER_BYTE_LENGTH);

  for (let hexIndex = 0, hexPairIndex = 0; hexIndex < threadCounter.length; hexIndex += 2, hexPairIndex++) {
    const hexPairString = threadCounter.substring(hexIndex, hexIndex + 2);

    _assert.default.strictEqual(hexPairString.length, 2);

    const byte = Buffer.from(hexPairString, 'hex');

    _assert.default.strictEqual(byte.length, 1, 'One-byte buffer expected from a pair of hex digits');

    buffer[hexPairIndex] = byte[0];
  }

  return buffer;
};

exports.threadCounterHexStringToBuffer = threadCounterHexStringToBuffer;

const threadCounterToBuffer = threadCounter => {
  return threadCounterHexStringToBuffer(threadCounterToHexString(threadCounter));
};

exports.threadCounterToBuffer = threadCounterToBuffer;

const emptyLoadEventsResult = cursor => {
  return {
    cursor: cursor == null ? threadArrayToCursor(initThreadArray()) : cursor,
    events: []
  };
};

exports.emptyLoadEventsResult = emptyLoadEventsResult;

const calculateMaxThreadArray = threadArrays => {
  const maxThreadArray = initThreadArray();

  for (const threadArray of threadArrays) {
    checkThreadArrayLength(threadArray);

    for (let i = 0; i < _constants.THREAD_COUNT; ++i) {
      maxThreadArray[i] = Math.max(maxThreadArray[i], threadArray[i]);
    }
  }

  return maxThreadArray;
};

const checkEventsContinuity = (startingCursor, eventCursorPairs) => {
  const startingThreadArray = cursorToThreadArray(startingCursor);
  const tuples = eventCursorPairs.map(({
    event,
    cursor
  }) => {
    return {
      event,
      cursor,
      threadArray: cursorToThreadArray(cursor)
    };
  });

  for (let i = 0; i < tuples.length; ++i) {
    _assert.default.strictEqual(tuples[i].event.threadCounter, tuples[i].threadArray[tuples[i].event.threadId] - 1);

    if (startingThreadArray[tuples[i].event.threadId] > tuples[i].event.threadCounter) {
      return false;
    }
  }

  const maxThreadArray = calculateMaxThreadArray(tuples.map(t => t.threadArray));

  for (const t of tuples) {
    startingThreadArray[t.event.threadId]++;
  }

  for (let i = 0; i < _constants.THREAD_COUNT; ++i) {
    if (maxThreadArray[i] !== startingThreadArray[i]) {
      return false;
    }
  }

  return true;
};

exports.checkEventsContinuity = checkEventsContinuity;
//# sourceMappingURL=cursor-operations.js.map